#!perl

use Test::More tests => 22;
use POSIX qw(strftime);
use Data::Dumper;

BEGIN {
  require "t/common.pl";
  start_server();
}

$ldap = client();
if (!$ldap) {
  die "can't contact ldap server";
}

$mesg = $ldap->bind($MANAGERDN, password => $PASSWD);

if ($mesg->code) {
  die "can't bind: ".$mesg->error;
}


# Add
ldif_populate($ldap, "data/01-in.ldif");

$mesg = $ldap->search(base => $SQLDN, filter => '(objectClass=*)');
compare_ldif("0101",$mesg,$mesg->sorted);


$dn = "uid=user0101,ou=Tech,ou=IT,ou=People,$SQLDN";

# Modify
$mesg = $ldap->modify($dn,
                        changes => [
                          add => ['telephoneNumber' => '23-4567-8901'],
                          add => ['businessCategory' => 'cn=Admin,ou=App,ou=Roles,ou=SQL,dc=lism,dc=com'],
                          delete => ['telephoneNumber' => '12-3456-7890'],
                          delete => ['businessCategory' => 'cn=Guest,ou=App,ou=Roles,ou=SQL,dc=lism,dc=com']
                       ]
                     );
$mesg = $ldap->search(base => $SQLDN, filter => 'uid=user0101');
compare_ldif("0102",$mesg,$mesg->sorted);

$mesg = $ldap->modify($dn,
                        changes => [
                          replace => ['cn' => '\'更新\'ユーザ0101'],
                          replace => ['mail' => 'user0101@lism.org'],
                          replace => ['telephoneNumber' => ['01-2345-6789', '34-5678-9012']],
                          replace => ['businessCategory' => ['cn=Admin,ou=App,ou=Roles,ou=SQL,dc=lism,dc=com', 'cn=Guest,ou=App,ou=Roles,ou=SQL,dc=lism,dc=com']]
                       ]
                     );
$mesg = $ldap->search(base => $SQLDN, filter => 'uid=user0101');
compare_ldif("0103",$mesg,$mesg->sorted);

$mesg = $ldap->modify($dn,
                        changes => [
			  delete => ['mail' => []],
                          delete => ['telephoneNumber' => []],
                          delete => ['businessCategory' => []]
                        ]
                     );
$mesg = $ldap->search(base => $SQLDN, filter => 'uid=user0101');
compare_ldif("0104",$mesg,$mesg->sorted);


# Search
$mesg = $ldap->search(base => $SQLDN, filter => 'objectClass=*', scope => 'base');
compare_ldif("0105",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => $SQLDN, filter => 'objectClass=*', scope => 'one', typesonly => 1);
compare_ldif("0106",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => "ou=People,$SQLDN", filter => 'objectClass=*', scope => 'base', attrs => ['ou']);
compare_ldif("0107",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => "ou=People,$SQLDN", filter => 'objectClass=*', scope => 'one');
compare_ldif("0108",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => "ou=Tech,ou=IT,ou=People,$SQLDN", filter => 'objectClass=*', scope => 'base');
compare_ldif("0109",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => "ou=Tech,ou=IT,ou=People,$SQLDN", filter => 'objectClass=*', scope => 'one', typesonly => 1);
compare_ldif("0110",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => "ou=Tech,ou=IT,ou=People,$SQLDN", filter => 'objectClass=*', attrs => ['objectClass', 'uid']);
compare_ldif("0111",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => "uid=user0101,ou=Tech,ou=IT,ou=People,$SQLDN", filter => 'objectClass=*', typesonly => 1, attrs => ['cn', 'userPassword']);
compare_ldif("0112",$mesg,$mesg->sorted);

$mesg = $ldap->search(base => "uid=user0101,ou=Tech,ou=IT,ou=People,$SQLDN", filter => 'cn=\'更新\'ユーザ0101', attrs => ['cn', 'modifyTimestamp']);
$timestamp = ($mesg->entries)[0]->get_value('modifyTimestamp');
$date = strftime("%Y%m%d", localtime);

ok($timestamp =~ /^$date.*Z$/, "modifyTimestamp");

# Compare
$mesg = $ldap->compare($dn, attr => 'cn', value => '\'更新\'ユーザ0101');
ok($mesg->code == 6, "compare uid=user0101 true");

$mesg = $ldap->compare($dn, attr => 'cn', value => '更新ユーザ0100');
ok($mesg->code == 5, "compare uid=user0101 false");


# Bind
$mesg = $ldap->bind($dn, password => 'user0101');
ok($mesg->code == 0, "bind uid=user0101 success");

$mesg = $ldap->bind($dn, password => 'user0100');
ok($mesg->code == 49, "bind uid=user0101 failure");

$ldap->bind($MANAGERDN, password => $PASSWD);

# Delete
$mesg = $ldap->delete($dn);
$mesg = $ldap->search(base => $SQLDN, filter => 'cn=user01');
ok($mesg->count == 0, "delete uid=user0101");


# Rollback
ldif_populate($ldap, "data/0113-in.ldif");
$mesg = $ldap->search(base => $SQLDN, filter => 'cn=user01');
ok($mesg->count == 0, "rollback uid=user0101");

# Plugin
ldif_populate($ldap, "data/0120-in.ldif");
ok(!compare("/tmp/output.txt","data/0120-output.txt"), "data/0120-output.txt");

$mesg = $ldap->modify($dn,
                        changes => [
                          replace => ['mail' => 'user0101@lism.org']
                       ]
                     );
ok(!compare("/tmp/output.txt","data/0121-output.txt"), "data/0121-output.txt");

# Delete
$mesg = $ldap->delete($dn);
ok(!compare("/tmp/output.txt","data/0122-output.txt"), "data/0122-output.txt");
